@@ -16,6 +16,14 @@ typedef int __int64_t;
 typedef int __uint64_t;
 typedef int __int_least32_t;
 typedef int __uint_least32_t;
+typedef int __s8;
+typedef int __u8;
+typedef int __s16;
+typedef int __u16;
+typedef int __s32;
+typedef int __u32;
+typedef int __s64;
+typedef int __u64;
 typedef int _LOCK_T;
 typedef int _LOCK_RECURSIVE_T;
 typedef int _off_t;
@@ -95,6 +103,7 @@ typedef int pthread_barrier_t;
 typedef int pthread_barrierattr_t;
 typedef int jmp_buf;
 typedef int rlim_t;
+typedef int sa_family_t;
 typedef int sigjmp_buf;
 typedef int stack_t;
 typedef int siginfo_t;
